==> wiiuse_v0.1_src.tar.gz: wiiuse/api/example.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *
 *	@brief Example using the wiiuse API.
 *
 *	This file is an example of how to use the wiiuse library.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "wiiuse.h"


/**
 *	@brief Callback that handles an event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param btns		What buttons are currently pressed.  They are OR'ed together.
 *	@param accel	Acceleration of the device along each axis.
 *					This is the raw data reported by the wiimote.
 *	@param orient	Orientation (roll, pitch, yaw) of the device.
 *	@param gforce	Pull of gravity on each axis of the device (measured in gravity units).
 *
 *	This function is called automatically by the wiiuse library when an
 *	event occurs on the specified wiimote.
 */
void handle_event(struct wiimote_t* wm, int btns, vec3b_t accel, orient_t orient, gforce_t gforce) {
	printf("\n\n--- EVENT [wiimote id %i] ---\n", wm->unid);
	if (btns) {
		/* if a button is pressed, report it */
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_A))		printf("A pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_B))		printf("B pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_UP))	printf("UP pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_DOWN))	printf("DOWN pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_LEFT))	printf("LEFT pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_RIGHT))	printf("RIGHT pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_MINUS))	printf("MINUS pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_PLUS))	printf("PLUS pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_ONE))	printf("ONE pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_TWO))	printf("TWO pressed\n");
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_HOME))	printf("HOME pressed\n");

		/*
		 *	Pressing minus will tell the wiimote we are no longer interested in movement.
		 *	This is useful because it saves battery power.
		 */
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_MINUS))
			wiimote_motion_sensing(wm, 0);

		/*
		 *	Pressing plus will tell the wiimote we are interested in movement.
		 */
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_PLUS))
			wiimote_motion_sensing(wm, 1);

		/*
		 *	Pressing B will toggle the rumble
		 *
		 *	if B is pressed but is not held, toggle the rumble
		 */
		if (IS_PRESSED(btns, WIIMOTE_BUTTON_B) && !IS_HELD(wm, WIIMOTE_BUTTON_B))
			wiimote_toggle_rumble(wm);
	}

	printf("Zero:\tX=%i\tY=%i\tZ=%i\n", wm->cal_zero.x, wm->cal_zero.y, wm->cal_zero.z);
	printf("  1g:\tX=%i\tY=%i\tZ=%i\n", wm->cal_g.x, wm->cal_g.y, wm->cal_g.z);
	printf(" Raw:\tX=%i\tY=%i\tZ=%i\n", accel.x, accel.y, accel.z);

	printf("gforce x: %f\n", gforce.x);
	printf("gforce y: %f\n", gforce.y);
	printf("gforce z: %f\n", gforce.z);

	printf("roll  = %f\n", orient.roll);
	printf("pitch = %f\n", orient.pitch);
	printf("yaw   = %f\n", orient.yaw);

}


/**
 *	@brief Callback that handles a read event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param data		Pointer to the filled data block.
 *	@param len		Length in bytes of the data block.
 *
 *	This function is called automatically by the wiiuse library when
 *	the wiimote has returned the full data requested by a previous
 *	call to wiimote_read_data().
 *
 *	You can read data on the wiimote, such as Mii data, if
 *	you know the offset address and the length.
 *
 *	The \a data pointer was specified on the call to wiimote_read_data().
 *	At the time of this function being called, it is not safe to deallocate
 *	this buffer.
 */
void handle_read(struct wiimote_t* wm, byte* data, unsigned short len) {
	printf("\n\n--- DATA READ [wiimote id %i] ---\n", wm->unid);
	printf("finished read of size %i\n", len);
	int i = 0;
	for (; i < len; ++i) {
		if (!(i%16))
			printf("\n");
		printf("%x ", data[i]);
	}
	printf("\n\n");
}


/**
 *	@brief Callback that handles a controller status event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *	@param attachment		Is there an attachment? (1 for yes, 0 for no)
 *	@param speaker			Is the speaker enabled? (1 for yes, 0 for no)
 *	@param ir				Is the IR support enabled? (1 for yes, 0 for no)
 *	@param led				What LEDs are lit.
 *	@param battery_level	Battery level, between 0.0 (0%) and 1.0 (100%).
 *
 *	This occurs when either the controller status changed
 *	or the controller status was requested explicitly by
 *	wiimote_status().
 *
 *	One reason the status can change is if the nunchuk was
 *	inserted or removed from the expansion port.
 */
void handle_ctrl_status(struct wiimote_t* wm, int attachment, int speaker, int ir, int led[4], float battery_level) {
	printf("\n\n--- CONTROLLER STATUS [wiimote id %i] ---\n", wm->unid);

	printf("attachment:      %i\n", attachment);
	printf("speaker:         %i\n", speaker);
	printf("ir:              %i\n", ir);
	printf("led 1:           %i\n", led[0]);
	printf("led 2:           %i\n", led[1]);
	printf("led 3:           %i\n", led[2]);
	printf("led 4:           %i\n", led[3]);
	printf("battery:         %f %%\n", battery_level);
}


/**
 *	@brief Callback that handles a disconnection event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *
 *	This can happen if the POWER button is pressed, or
 *	if the connection is interrupted.
 */
void handle_disconnect(wiimote* wm) {
	printf("\n\n--- DISCONNECTED [wiimote id %i] ---\n", wm->unid);
}


/*
 *	These are some identifiers for wiimotes
 *
 *	See below in main() for what they are used for.
 */
#define WIIMOTE_ID_1		1
#define WIIMOTE_ID_2		2


/**
 *	@brief main()
 *
 *	Connect to up to two wiimotes and print any events
 *	that occur on either device.
 */
int main(int argc, char** argv) {
	/*
	 *	Load the wiiuse library
	 *
	 *	This needs to be done before anything else can happen
	 */
	float v = wiiuse_startup("../src/wii.so");
	printf("v = %f\n", v);
	if (!v) {
		fprintf(stderr, "Failed to load wiiuse library.\n");
		return 0;
	}

	/*
	 *	Make a temp array of wiimote ids.
	 *
	 *	Here I only anticipate connecting up to
	 *	two wiimotes.  Each wiimote connected
	 *	will get one of these ids.
	 */
	int ids[] = { WIIMOTE_ID_1, WIIMOTE_ID_2 };

	/*
	 *	Initialize an array of wiimote objects.
	 *
	 *	The first parameter is the number of wiimotes
	 *	I want to create.  I only have two wiimotes
	 *	so I'm limiting the test to just 2.
	 *
	 *	Then I get it the array of ids and a couple
	 *	callback functions to invoke when something
	 *	happens on one of the wiimotes.
	 *
	 *	handle_event gets called when a generic event occurs (button press, motion sensing, etc)
	 *	handle_ctrl_status gets called when a response to a status request arrives (battery power, etc)
	 *	handle_disconnect gets called when the wiimote disconnect (holding power button)
	 */
	wiimote** wiimotes =  wiimote_init(2, ids, handle_event, handle_ctrl_status, handle_disconnect);

	/*
	 *	Find wiimote devices
	 *
	 *	Now we need to find some wiimotes.
	 *	Give the function the wiimote array we created, and tell it there
	 *	are 2 wiimotes we are interested in.
	 *
	 *	Set the timeout to be 5 seconds.
	 *
	 *	This will return the number of actual wiimotes that are in discovery mode.
	 */
	int found = wiimote_find(wiimotes, 2, 5);
	if (!found)
		return 0;

	/*
	 *	Connect to the wiimotes
	 *
	 *	Now that we found some wiimotes, connect to them.
	 *	Give the function the wiimote array and the number
	 *	of wiimote devices we found.
	 *
	 *	This will return the number of established connections to the found wiimotes.
	 */
	int connected = wiimote_connect(wiimotes, 2);
	if (connected)
		printf("Connected to %i wiimotes (of %i found).\n", connected, found);
	else {
		printf("Failed to connect to any wiimote.\n");
		return 0;
	}

	/*
	 *	Now set the LEDs and rumble for a second so it's easy
	 *	to tell which wiimotes are connected (just like the wii does).
	 */
	wiimote_set_leds(wiimotes[0], WIIMOTE_LED_1 | WIIMOTE_LED_4);
	wiimote_set_leds(wiimotes[1], WIIMOTE_LED_2 | WIIMOTE_LED_3);
	wiimote_rumble(wiimotes[0], 1);
	wiimote_rumble(wiimotes[1], 1);
	usleep(200000);
	wiimote_rumble(wiimotes[0], 0);
	wiimote_rumble(wiimotes[1], 0);

	/*
	 *	Maybe I'm interested in the battery power of the 0th
	 *	wiimote.  This should be WIIMOTE_ID_1 but to be sure
	 *	you can get the wiimote assoicated with WIIMOTE_ID_1
	 *	using the wiimote_get_by_id() function.
	 *
	 *	A status request will return other things too, like
	 *	if any extensions are plugged into the wiimote or
	 *	what LEDs are lit.
	 */
	wiimote_status(wiimotes[0]);

	/*
	 *	This is the main loop
	 *
	 *	wiimote_poll() needs to be called with the wiimote array
	 *	and the number of wiimote structures in that array
	 *	(it doesn't matter if some of those wiimotes are not used
	 *	or are not connected).
	 *
	 *	This function will invoke the callbacks set in wiimote_init()
	 *	when the wiimote has things to report.
	 */
	while (1) {
		wiimote_poll(wiimotes, 2);
	}

	/*
	 *	Disconnect the wiimotes
	 */
	wiimote_disconnect(wiimotes[0]);
	wiimote_disconnect(wiimotes[1]);

	/*
	 *	Unload the wiiuse library
	 */
	wiiuse_shutdown();

	return 0;
}

==> wiiuse_v0.2_src.tar.gz: wiiuse/api/example.c <==
/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2005
 *
 *	This file is part of wiiuse.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *	$Header$
 *
 */

/**
 *	@file
 *
 *	@brief Example using the wiiuse API.
 *
 *	This file is an example of how to use the wiiuse library.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "wiiuse.h"


/**
 *	@brief Callback that handles an event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *
 *	This function is called automatically by the wiiuse library when an
 *	event occurs on the specified wiimote.
 */
void handle_event(struct wiimote_t* wm) {
	printf("\n\n--- EVENT [wiimote id %i] ---\n", wm->unid);
	if (wm->btns) {
		/* if a button is pressed, report it */
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_A))		printf("A pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_B))		printf("B pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_UP))		printf("UP pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_DOWN))	printf("DOWN pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_LEFT))	printf("LEFT pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_RIGHT))	printf("RIGHT pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_MINUS))	printf("MINUS pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_PLUS))	printf("PLUS pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_ONE))		printf("ONE pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_TWO))		printf("TWO pressed\n");
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_HOME))	printf("HOME pressed\n");

		/*
		 *	Pressing minus will tell the wiimote we are no longer interested in movement.
		 *	This is useful because it saves battery power.
		 */
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_MINUS))
			wiimote_motion_sensing(wm, 0);

		/*
		 *	Pressing plus will tell the wiimote we are interested in movement.
		 */
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_PLUS))
			wiimote_motion_sensing(wm, 1);

		/*
		 *	Pressing B will toggle the rumble
		 *
		 *	if B is pressed but is not held, toggle the rumble
		 */
		if (IS_PRESSED(wm, WIIMOTE_BUTTON_B) && !IS_HELD(wm, WIIMOTE_BUTTON_B))
			wiimote_toggle_rumble(wm);
	}

	printf("wiimote roll  = %f\n", wm->orient.roll);
	printf("wiimote pitch = %f\n", wm->orient.pitch);
	printf("wiimote yaw   = %f\n", wm->orient.yaw);

	if (wm->expansion.type == EXP_NUNCHUK) {
		struct nunchuk_t* nc = &wm->expansion.nunchuk;

		if (IS_PRESSED(nc, NUNCHUK_BUTTON_C))		printf("C pressed\n");
		if (IS_PRESSED(nc, NUNCHUK_BUTTON_Z))		printf("Z pressed\n");

		printf("nunchuk roll  = %f\n", nc->orient.roll);
		printf("nunchuk pitch = %f\n", nc->orient.pitch);
		printf("nunchuk yaw   = %f\n", nc->orient.yaw);

		printf("nunchuk joystick angle:     %f\n", nc->js.ang);
		printf("nunchuk joystick magnitude: %f\n", nc->js.mag);
	}
}


/**
 *	@brief Callback that handles a read event.
 *
 *	@param wm		Pointer to a wiimote_t structure.
 *	@param data		Pointer to the filled data block.
 *	@param len		Length in bytes of the data block.
 *
 *	This function is called automatically by the wiiuse library when
 *	the wiimote has returned the full data requested by a previous
 *	call to wiimote_read_data().
 *
 *	You can read data on the wiimote, such as Mii data, if
 *	you know the offset address and the length.
 *
 *	The \a data pointer was specified on the call to wiimote_read_data().
 *	At the time of this function being called, it is not safe to deallocate
 *	this buffer.
 */
void handle_read(struct wiimote_t* wm, byte* data, unsigned short len) {
	printf("\n\n--- DATA READ [wiimote id %i] ---\n", wm->unid);
	printf("finished read of size %i\n", len);
	int i = 0;
	for (; i < len; ++i) {
		if (!(i%16))
			printf("\n");
		printf("%x ", data[i]);
	}
	printf("\n\n");
}


/**
 *	@brief Callback that handles a controller status event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *	@param attachment		Is there an attachment? (1 for yes, 0 for no)
 *	@param speaker			Is the speaker enabled? (1 for yes, 0 for no)
 *	@param ir				Is the IR support enabled? (1 for yes, 0 for no)
 *	@param led				What LEDs are lit.
 *	@param battery_level	Battery level, between 0.0 (0%) and 1.0 (100%).
 *
 *	This occurs when either the controller status changed
 *	or the controller status was requested explicitly by
 *	wiimote_status().
 *
 *	One reason the status can change is if the nunchuk was
 *	inserted or removed from the expansion port.
 */
void handle_ctrl_status(struct wiimote_t* wm, int attachment, int speaker, int ir, int led[4], float battery_level) {
	printf("\n\n--- CONTROLLER STATUS [wiimote id %i] ---\n", wm->unid);

	printf("attachment:      %i\n", attachment);
	printf("speaker:         %i\n", speaker);
	printf("ir:              %i\n", ir);
	printf("leds:            %i %i %i %i\n", led[0], led[1], led[2], led[3]);
	printf("battery:         %f %%\n", battery_level);
}


/**
 *	@brief Callback that handles a disconnection event.
 *
 *	@param wm				Pointer to a wiimote_t structure.
 *
 *	This can happen if the POWER button is pressed, or
 *	if the connection is interrupted.
 */
void handle_disconnect(wiimote* wm) {
	printf("\n\n--- DISCONNECTED [wiimote id %i] ---\n", wm->unid);
}


/*
 *	These are some identifiers for wiimotes
 *
 *	See below in main() for what they are used for.
 */
#define WIIMOTE_ID_1		1
#define WIIMOTE_ID_2		2


/**
 *	@brief main()
 *
 *	Connect to up to two wiimotes and print any events
 *	that occur on either device.
 */
int main(int argc, char** argv) {
	/*
	 *	Load the wiiuse library
	 *
	 *	This needs to be done before anything else can happen
	 *	wiiuse_startup() will return the version of the library loaded.
	 */
	float version = wiiuse_startup("../src/wii.so");
	printf("Wiiuse Version = %f\n", version);
	if (!version) {
		fprintf(stderr, "Failed to load wiiuse library.\n");
		return 0;
	}

	/*
	 *	Make a temp array of wiimote ids.
	 *
	 *	Here I only anticipate connecting up to
	 *	two wiimotes.  Each wiimote connected
	 *	will get one of these ids.
	 */
	int ids[] = { WIIMOTE_ID_1, WIIMOTE_ID_2 };

	/*
	 *	Initialize an array of wiimote objects.
	 *
	 *	The first parameter is the number of wiimotes
	 *	I want to create.  I only have two wiimotes
	 *	so I'm limiting the test to just 2.
	 *
	 *	Then I get it the array of ids and a couple
	 *	callback functions to invoke when something
	 *	happens on one of the wiimotes.
	 *
	 *	handle_event gets called when a generic event occurs (button press, motion sensing, etc)
	 *	handle_ctrl_status gets called when a response to a status request arrives (battery power, etc)
	 *	handle_disconnect gets called when the wiimote disconnect (holding power button)
	 */
	wiimote** wiimotes =  wiimote_init(2, ids, handle_event, handle_ctrl_status, handle_disconnect);

	/*
	 *	Find wiimote devices
	 *
	 *	Now we need to find some wiimotes.
	 *	Give the function the wiimote array we created, and tell it there
	 *	are 2 wiimotes we are interested in.
	 *
	 *	Set the timeout to be 5 seconds.
	 *
	 *	This will return the number of actual wiimotes that are in discovery mode.
	 */
	int found = wiimote_find(wiimotes, 2, 5);
	if (!found)
		return 0;

	/*
	 *	Connect to the wiimotes
	 *
	 *	Now that we found some wiimotes, connect to them.
	 *	Give the function the wiimote array and the number
	 *	of wiimote devices we found.
	 *
	 *	This will return the number of established connections to the found wiimotes.
	 */
	int connected = wiimote_connect(wiimotes, 2);
	if (connected)
		printf("Connected to %i wiimotes (of %i found).\n", connected, found);
	else {
		printf("Failed to connect to any wiimote.\n");
		return 0;
	}

	/*
	 *	Now set the LEDs and rumble for a second so it's easy
	 *	to tell which wiimotes are connected (just like the wii does).
	 */
	wiimote_set_leds(wiimotes[0], WIIMOTE_LED_1 | WIIMOTE_LED_4);
	wiimote_set_leds(wiimotes[1], WIIMOTE_LED_2 | WIIMOTE_LED_3);
	wiimote_rumble(wiimotes[0], 1);
	wiimote_rumble(wiimotes[1], 1);
	usleep(200000);
	wiimote_rumble(wiimotes[0], 0);
	wiimote_rumble(wiimotes[1], 0);

	/*
	 *	Maybe I'm interested in the battery power of the 0th
	 *	wiimote.  This should be WIIMOTE_ID_1 but to be sure
	 *	you can get the wiimote assoicated with WIIMOTE_ID_1
	 *	using the wiimote_get_by_id() function.
	 *
	 *	A status request will return other things too, like
	 *	if any extensions are plugged into the wiimote or
	 *	what LEDs are lit.
	 */
	wiimote_status(wiimotes[0]);

	/*
	 *	This is the main loop
	 *
	 *	wiimote_poll() needs to be called with the wiimote array
	 *	and the number of wiimote structures in that array
	 *	(it doesn't matter if some of those wiimotes are not used
	 *	or are not connected).
	 *
	 *	This function will invoke the callbacks set in wiimote_init()
	 *	when the wiimote has things to report.
	 */
	while (1) {
		wiimote_poll(wiimotes, 2);
	}

	/*
	 *	Disconnect the wiimotes
	 */
	wiimote_disconnect(wiimotes[0]);
	wiimote_disconnect(wiimotes[1]);

	/*
	 *	Unload the wiiuse library
	 */
	wiiuse_shutdown();

	return 0;
}
